function [ IMSE ] = IMSE( S, rho )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IMSE:
% This function calculates the IMSE (normalized for the variance) for any
% number of dimensions of rhos in the simulator only problem when given
% a particular design matrix S (nxd) and a particular vector draw of rho.
%
% Authors: Erin Leatherman, West Virginia University
%
% Questions/Comments: 
% Please email Erin Leatherman at erleatherman@mail.wvu.edu.
%
% Copyright 2016. Erin Leatherman, West Virginia University.
%
% THERE IS NO WARRANTY, EXPRESS OR IMPLIED. WE DO NOT ASSUME ANY LIABILITY
% FOR THE USE OF THIS SOFTWARE.  If software is modified to produce
% derivative works, such modified software should be clearly marked.
% Additionally, this program is free software; you can redistribute it 
% and/or modify it under the terms of the GNU General Public License as 
% published by the Free Software Foundation; version 2.0 of the License. 
% Accordingly, this program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
% of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
% General Public License for more details.
%
% For function details and reference information, see:
% http://stat.wvu.edu/~erl/CompExpDesgs_Pred/designs/
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUTS:
% S = nxd matrix of design points, where 
%       n=number of design points
%       d=number of input variables
% rho = 1xd row vector of correlation parameters 
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUT: 
% IMSE = (1/\sigma_Z^2) \times the integrated mean squared prediction
%      error of the BLUP (for the GP(\beta, \sigma_Z^2,\rho_vector) 
%      stochastic model)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% REQUIRED FUNCTIONS: S1_0.m, S2.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Check to see if correlation matrix is of the form ones(m,m)
% Return IMSE=NA if rho==ones(1,d), otherwise continue
if rho==ones(1,size(rho,2))
    IMSE=NaN;
    str = sprintf('*** Warning: Correlation matrix only contains 1s! ***');
    disp(str);
else


%% Calculate block matrix, M, used in the calculation of IMSE
[Si Sj]=meshgrid(1:size(S,1),1:size(S,1));
dsq=4*(S(Si,:)-S(Sj,:)).^2;
R=reshape(prod(bsxfun(@power,rho,dsq),2),size(S,1),size(S,1));
F=ones(size(S,1),1);    % uses constant-mean model
M=[0 F'; F R];


%% Calculate integrand in the calculation of the IMSE
gamma=-4*log(rho);

rhoprod=prod(rho);

%integrate ff' dx (uses constant-mean model)
A=1;

%integrate vf' dx
switch rhoprod
    case 0
        C=zeros(size(S,1),1);
    otherwise
        C=prod(S1_0(S(:,(gamma~=0)),gamma(gamma~=0)),2);
end

%integrate fv' dx
B=C';

%integrate vv' dx
switch rhoprod
    case 0
        D=zeros(size(S,1),size(S,1));
    otherwise
        D=S2(S(:,(gamma~=0)),gamma(gamma~=0));
end


%% find the matrix inside the trace
tmp=M\[A B; C D];

%% Calculate IMSE
IMSE=(1-trace(tmp));

end
end

