function [ PSO, fmin] = IMSPEopt_simulator(n, d, rho, Ndes, Nits, id)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IMSPEopt_simulator:
% This function calculates an IMSPE-optimal simulator design with n runs
% in d dimensions using a combination of Particle Swarm Optimization (PSO)
% and Quasi-Newton optimization.
%
% Author: Erin Leatherman, West Virginia University
%
% Questions/Comments: 
% Please email Erin Leatherman at erleatherman@mail.wvu.edu.
%
% Copyright 2016. Erin Leatherman, West Virginia University.
%
% THERE IS NO WARRANTY, EXPRESS OR IMPLIED. WE DO NOT ASSUME ANY LIABILITY
% FOR THE USE OF THIS SOFTWARE.  If software is modified to produce
% derivative works, such modified software should be clearly marked.
% Additionally, this program is free software; you can redistribute it 
% and/or modify it under the terms of the GNU General Public License as 
% published by the Free Software Foundation; version 2.0 of the License. 
% Accordingly, this program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
% of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
% General Public License for more details.
%
% For function details and reference information, see:
% http://stat.wvu.edu/~erl/CompExpDesgs_Pred/designs/
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUTS:
% n = number of simulator runs
% d = number of input dimensions
% rho = 1xd row vector of simulator correlation parameters
% Ndes = number of designs (particles) for PSO
% Nits = number of iterations for PSO
% id = character string to identify which example is being run
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUTS: 
% PSO = structure containing the outputs from the PSO part of the
%       optimization routine, with elements
%	PSO.IMSPE = best IMSPE value found using PSO
%   PSO.Xbest = IMSPE-optimal simulator design found using PSO only
%   PSO.Xbest_vect = PSO.Xbest reshaped into a vector of length nd x 1
%   PSO.PSOtime = time elapsed during PSO optimization
%
% fmin = structure containing the outputs from the fmincon part of the
%       optimization routine, with elements
%   fmin.IMSPE = best IMSPE value found using fmincon.m
%   fmin.Xbest = IMSPE-optimal simulator design found using PSO + fmincon
%   fmin.exitflag = exit flag describing why fmincon search was terminated 
%   fmin.output = output from fmincon.m function
%   fmin.fmintime = time elapsed during fmincon.m optimization
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Required Functions: PSO_IMSPE_Sim.m, vect_IMSE.m, IMSE.m, S1_0.m, S2.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% put user-specified information in global variable format
global nglob dglob sdraws
nglob = n;
dglob = d;
sdraws = rho;

%%% give additional parameters needed for PSO
beta = 2;
alpha = 2;
gamma = 0.5;

%%% run PSO
PSO = PSO_IMSPE_Sim( Ndes, Nits, beta, alpha, gamma, id);

%%% save results
save(strcat('out_',id))

%%% run fmincon
x0=[PSO.Xbest_vect];

Amat=[eye(nglob*dglob); -1*eye(nglob*dglob)];
bvect=[ones(nglob*dglob,1); zeros(nglob*dglob,1)];

options=optimset('Algorithm','active-set');
options.MaxFunEvals=18000;

tic
[x,fval,exitflag,output]=fmincon(@vect_IMSE,x0,Amat,bvect,[],[],...
	zeros(nglob*dglob,1),ones(nglob*dglob,1),[],options);
fmintime = toc;

fmin.IMSPE=fval;
fmin.Xbest=reshape(x,nglob,dglob);
fmin.exitflag=exitflag;
fmin.output=output;
fmin.fmintime = fmintime;

clearvars -global

end

