function [ Op ] = PSO_IMSPE_Sim( Ndes, Nits, beta, alpha, gamma, id)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PSO_IMSPE_Sim:
% This function is used within the IMSPEopt_simulator.m code.  This code
% calculates the IMSPE-optimal design using Particle Swarm Optimization.
%
% Author: Erin Leatherman, West Virginia University
%
% Questions/Comments: 
% Please email Erin Leatherman at erleatherman@mail.wvu.edu.
%
% Copyright 2016. Erin Leatherman, West Virginia University.
%
% THERE IS NO WARRANTY, EXPRESS OR IMPLIED. WE DO NOT ASSUME ANY LIABILITY
% FOR THE USE OF THIS SOFTWARE.  If software is modified to produce
% derivative works, such modified software should be clearly marked.
% Additionally, this program is free software; you can redistribute it 
% and/or modify it under the terms of the GNU General Public License as 
% published by the Free Software Foundation; version 2.0 of the License. 
% Accordingly, this program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
% of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
% General Public License for more details.
%
% For function details and reference information, see:
% http://stat.wvu.edu/~erl/CompExpDesgs_Pred/designs/
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUTS:
% Ndes = number of designs (particles) for PSO
% Nits = number of iterations for PSO
% beta = weight put on the step toward the personal-best position
% alpha = weight put on the step toward the global-best position
% gamma = weight put on velocity from previous iteration
% id = character string to identify which example is being run
%
% GLOBAL INPUTS:
% nglob = number of simulator runs
% dglob = number of input dimensions
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUT: 
% Op = structure containing the outputs from the PSO part of the
%       optimization routine, with elements
%	Op.IMSPE = best IMSPE value found using PSO
%   Op.Xbest = IMSPE-optimal simulator design found using PSO only
%   Op.Xbest_vect = PSO.Xbest reshaped into a vector of length n*d
%   Op.PSOtime = time elapsed during PSO optimization
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Required Functions: vect_IMSE.m, IMSE.m, S1_0.m, S2.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic

global nglob dglob

%set up design matrices
X=NaN(nglob*dglob,Ndes,Nits);  %initial size of matrix to store all designs

%start with an (Ndes \times m*d) design that covers the space well
X(:,:,1)=[lhsdesign(Ndes,nglob*dglob,...
    'criterion','maximin','iterations', 5000)]';


%calculate IMSPE for each initial particle
Ival=NaN(Ndes,Nits);
for i=1:Ndes
    Ival(i,1)=vect_IMSE( X(:,i,1) );
end

f_pbest=NaN(Ndes,Nits);
f_pbest(:,1)=Ival(:,1);
ind_p=ones(Ndes,1);

f_gbest=NaN(Nits,1);
[f_gbest(1), ind_g]=min(f_pbest(:,1));
ind_g(1,2)=1;


%% run for 'its' iterations
%initialize velocity vector
v = zeros(size(X,1), size(X,2));

for t=2:Nits
    %grab global best location
    Xg=X(:,ind_g(1),ind_g(2));
    
    %grab particle best locations
    for i=1:Ndes
        Xp(:,i)=X(:,i,ind_p(i));
    end
        
    %update velocity
    v = gamma*v + ...
        alpha*unifrnd(0,1,[size(v)]).*bsxfun(@plus, -X(:,:,t-1),Xg) + ...
        beta*unifrnd(0,1,[size(v)]).*(Xp-X(:,:,t-1));
    
    %give a maximum abs(value) for the velocity
    vmax = 1/4;
    v = (v <= 0).*max(v, -vmax*ones(size(v))) + (v > 0).*min(v, vmax*ones(size(v)));
    
    % update the particle locations
    tmp = v + X(:,:,t-1);
    
    %put any points outside of the boundary on the boundary
    tmp2=(tmp.*(tmp>=0)).*(tmp<=1)+(tmp>1);
    %take this to be the new design
    X(:,:,t)=tmp2;    
        
   
    %calculate the BIMSPE for each new location
    for i=1:Ndes
    Ival(i,t)=vect_IMSE( X(:,i,t) ); 
    end
    
    %update storage
    [f_pbest(:,t),ind_p]=min(Ival(:,1:t),[],2);
    [f_gbest(t), ind_g(1)]=min(f_pbest(:,t));
    ind_g(2)=ind_p(ind_g(1));
end
PSOtime = toc;

% save the answers
Op.IMSPE = f_gbest(t);
Op.Xbest = reshape(X(:,ind_g(1),ind_g(2)),nglob,dglob);
Op.Xbest_vect = X(:,ind_g(1),ind_g(2));
Op.PSOtime = PSOtime;


end

